#! /usr/bin/env sh
tether_pkgs='kmod-usb-net-rndis kmod-usb-net-cdc-eem kmod-usb-net-cdc-subset kmod-usb-net-huawei-cdc-ncm kmod-usb-net-cdc-ncm kmod-usb-net-cdc-ether kmod-usb-net-ipheth kmod-usb-net kmod-usb2 usbmuxd libimobiledevice usbutils'
install_tether_nws() {
  if ! uci show network.tether_eth2 2>/dev/null 1>&2
  then
    echo 'Adding network tether_eth2...'
    uci batch <<EOF
      set network.tether_eth2=interface
      set network.tether_eth2.proto=dhcp
      set network.tether_eth2.device=eth2
      set network.tether_eth2.metric=1190
EOF
  else echo 'Network tether_eth2 already exists.'
  fi
  if ! uci show network.tether_usb0 2>/dev/null 1>&2
  then
    echo 'Adding network tether_usb0...'
    uci batch <<EOF
      set network.tether_usb0=interface
      set network.tether_usb0.proto=dhcp
      set network.tether_usb0.device=usb0
      set network.tether_usb0.metric=1180
EOF
  else echo 'Network tether_usb0 already exists.'
  fi
  uci commit
}
uninstall_tether_nws() {
  uci show network.tether_eth2 2>/dev/null 1>&2
  if [ $? -eq 0 ]
  then
    echo 'Removing network tether_eth2...'
    uci del network.tether_eth2
  else echo 'Network tether_eth2 nonexistent.'
  fi
  uci show network.tether_usb0 2>/dev/null 1>&2
  if [ $? -eq 0 ]
  then
    echo 'Removing network tether_usb0...'
    uci del network.tether_usb0
  else echo 'Network tether_usb0 nonexistent.'
  fi
  uci commit
}
wan_zones() {
  uci show firewall | grep -E "@zone\[[0-9]+\].name='.+?wan(6|)'" | sort -ru | cut -d. -f2
}
install_tether_fws() {
  wan_zones | while read zone
  do
    for nw in tether_eth2 tether_usb0
    do
      uci show "firewall.$zone.network" | grep -q "$nw"
      if [ $? -ne 0 ]
      then
        echo "Adding network $nw to firewall zone $zone..."
        uci add_list "firewall.$zone.network=$nw"
      else echo "Network $nw already in firewall zone $zone."
      fi
    done
  done
  uci commit
}
uninstall_tether_fws() {
  wan_zones | while read zone
  do
    for nw in tether_eth2 tether_usb0
    do
      uci show "firewall.$zone.network" | grep -q "$nw"
      if [ $? -eq 0 ]
      then
        echo "Removing network $nw from firewall zone $zone..."
        uci del_list "firewall.$zone.network=$nw"
      else echo "Network $nw not in firewall zone $zone."
      fi
    done
  done
  uci commit
}
usage() {
   cat <<EOF
Install or uninstall support for tethering Android and iOS devices. Intended
for routers that do not ship with built-in tethering support. Most Gl.iNet
devices already have this support built-in. Vanilla OpenWrt devices might not.

$0 COMMAND [OPTIONS]

Commands:
  install
    A plethora of kernel modules and packages are required for this to work on
    a broad range of devices. The 'install' option adds all of them.
    Plugging in an Android device will typically use (or create, if nonexistent)
    the network device at 'usb0'. Apple iOS devices are similar, but prefer 'eth2'.
    The 'install' option will add the 'usb0' and 'eth2' devices into UCI as networks
    and give them membership in the WAN firewall zone.

  uninstall
    Removes the packages, network devices and firewall rules added during an 'install'.

  help, --help, -h
    Show this help message and exit.

Options:
  -f
    Force the installation (or uninstallation) of tethering support on a device
    which may already have support built-in.
EOF
}
warn_snippet_glinet_board() {
  cat <<EOF
This script is intended for devices without built-in tethering support.
Gl.iNet devices typically have built-in tethering support. If this device
is running vanilla OpenWrt (not Gl.iNet's firmware), you can continue safely.
EOF
}
is_glinet() {
  ubus call system board | grep -qE 'board_name.*glinet' 1> /dev/null 2>& 1 && grep -q glinet /etc/opkg/distfeeds.conf 1> /dev/null 2>& 1
}
read_eyn() {
  read -p '[Enter/y/N] ' eyn
  [ "$eyn" = '' ] || [ "$eyn" = 'y' ] || [ "$eyn" = 'Y' ]
}
case "$1" in
  help|--help|-h)
    usage
    ;;
  install)
    is_glinet
    if [ $? -eq 0 ] && ! echo "$@" | grep -q -- -f
    then
      if [ -t 0 ]
      then
        warn_snippet_glinet_board
        echo -n 'Continue? '
        read_eyn || exit 1
      else
        warn_snippet_glinet_board
        echo "Pass '-f' to force tethering support un/installation."
        exit 1
      fi
    fi
    opkg update
    opkg install $tether_pkgs
    /etc/init.d/usbmuxd enable
    /etc/init.d/usbmuxd start
    install_tether_nws
    install_tether_fws
    ;;
  uninstall)
    is_glinet
    if [ $? -eq 0 ] && ! echo "$@" | grep -q -- -f
    then
      if [ -t 0 ]
      then
        warn_snippet_glinet_board
        echo -n 'Continue? '
        read_eyn || exit 1
      else
        warn_snippet_glinet_board
        echo "Pass '-f' to force tethering support un/installation."
        exit 1
      fi
    fi
    uninstall_tether_fws
    uninstall_tether_nws
    opkg remove $tether_pkgs
    ;;
  *)
    usage
    exit 1
    ;;
esac
